#PART start

    float dt_ur_ms = DT * 100.0f;

    float h = READ_H;
    float n = READ_N;
    float b = READ_B;
    float z = READ_Z;

#PART middle

    // All numerical constants are unchanged from Golomb & Amitai 1997.

    float inf, tau;

    // Fast sodium.

    const float th_m   = -30.0f;
    const float rho_m  = 9.5f;
    const float th_h   = -53.0f;
    const float rho_h  = -7.0f;
    const float th_ht  = -40.5f;
    const float rho_ht = -6.0f;

    float m_inf = 1.0f / (1.0f + exp(-(v_m - th_m) / rho_m));

    inf = 1.0f / (1.0f + exp(-(v_m - th_h) / rho_h));
    tau = 0.37f + 2.78f / (1.0f + exp(-(v_m - th_ht) / rho_ht));
    h += (inf - h) / tau * dt_ur_ms;

    // Persistent sodium.

    const float th_p  = -40.0f;
    const float rho_p = 5.0f;
    
    float p_inf = 1.0f / (1.0f + exp(-(v_m - th_p) / rho_p));
    
    // Delayed rectifier potassium.

    const float th_n   = -30.0f;
    const float rho_n  = 10.0f;
    const float th_nt  = -27.0f;
    const float rho_nt = -15.0f;

    inf = 1.0f / (1.0f + exp(-(v_m - th_n) / rho_n));
    tau = 0.37f + 1.85f / (1.0f + exp(-(v_m - th_nt) / rho_nt));
    n += (inf - n) / tau * dt_ur_ms;

    // A-type potassium.

    const float th_a  = -50.0f;
    const float rho_a = 20.0f;
    const float th_b  = -80.0f;
    const float rho_b = -6.0f;
    const float tau_b = 15.0f;

    float a_inf = 1.0f / (1.0f + exp(-(v_m - th_a) / rho_a));

    inf = 1.0f / (1.0f + exp(-(v_m - th_b) / rho_b));
    b += (inf - b) / tau_b * dt_ur_ms;

    // Slow potassium.

    const float th_z  = -39.0f;
    const float rho_z = 5.0f;
    const float tau_z = 75.0f;

    inf = 1.0f / (1.0f + exp(-(v_m - th_z) / rho_z));
    z += (inf - z) / tau_z * dt_ur_ms;

    i_tot += G_NABAR  * (v_m - E_NA) * m_inf * m_inf * m_inf * h;
    i_tot += G_NAPBAR * (v_m - E_NA) * p_inf;
    i_tot += G_KDRBAR * (v_m - E_K ) * n * n * n * n;
    i_tot += G_KABAR  * (v_m - E_K ) * a_inf * a_inf * a_inf * b;
    i_tot += G_KSBAR  * (v_m - E_K ) * z;

#PART end

    WRITE_H(h);
    WRITE_N(n);
    WRITE_B(b);
    WRITE_Z(z);
