#BLOCKSIZE 16 8

// Total synaptic conductance at each tenth-step (uS).
#ARRAY syn_g 10

// Total synaptic current at each tenth-step if V_m = 0 (nA).
#ARRAY syn_i 10

#TEMPLATE

float i_inj;
if ((READ_TION <= ITER_NO) && (ITER_NO < READ_TIOFF)) {
    i_inj = READ_IINJ;
} else {
    i_inj = 0.0f;
}

float dt_ur_s = DT * 0.1f;

#UNROLL_START 10 %i
    SYN_G(%i) = 0.0f;
    SYN_I(%i) = 0.0f;
#UNROLL_END

for (int sn = 0; sn < NUM_SYN; sn++) {

    SELECT_SYN(sn);

    float g = READ_G;

    if (SYN_TYPE < 10) {

        // True synapse.

        float a = ALPHA(SYN_TYPE);
        float b = BETA (SYN_TYPE);
        float e = E_SYN(SYN_TYPE);

        float r = READ_R;

        int t_last = READ_PRE_T_LAST;

        float a2 = (ITER_NO - SPK_DUR < t_last) ? a : 0.0f;

        #UNROLL_START 10 %i
            r += dt_ur_s * (a2 * (1.0f - r) - b * r);
            SYN_G(%i) += r * g;
            SYN_I(%i) -= r * g * e;
        #UNROLL_END

        WRITE_R(r);

    } else {

        // Adjacent compartment.

        float e = READ_PRE_V_M;

        #UNROLL_START 10 %i
            SYN_G(%i) += g;
            SYN_I(%i) -= g * e;
        #UNROLL_END

    }

}

float v_m = READ_V_M;
float v_m_old = v_m;

#PART start

#UNROLL_START 10 %i

    float i_tot = -i_inj;

    i_tot += G_L * (v_m - E_L);

    #PART middle

    i_tot += SYN_G(%i) * v_m + SYN_I(%i);

    v_m += dt_ur_s * (-i_tot / C_M);

#UNROLL_END

#PART end

WRITE_V_M(v_m);

if ((v_m_old < E_SPK) && (E_SPK <= v_m)) {
    WRITE_T_LAST(ITER_NO);
}
